#pragma language glsl3

//camera input
uniform vec3 cam_pos;
uniform vec3 cam_forward;
uniform vec3 cam_up;
uniform float cam_minDist;
uniform float cam_maxDist;
uniform float cam_FOV;
uniform float cam_aspectRatio;

//per-vertex input
uniform vec3 vtx_pos[2];

#ifdef VERTEX
vec4 position( mat4 transform_projection, vec4 vertex_position )
{
    int i = int(vertex_position.x);//to get around the 2d limit of love.graphics, use the 2d info to index the 3d info

    //camera matrix stuff
    vec3 right = normalize(cross(cam_forward,cam_up));
    mat4 cam_matrix = mat4(
      right[0],right[1],right[2],0.0,
      cam_up[0],cam_up[1],cam_up[2],0.0,
      cam_forward[0],cam_forward[1],cam_forward[2],0.0,
      cam_pos[0], cam_pos[1], cam_pos[2],1.0
    );
    cam_matrix = inverse(cam_matrix);

    //projection matrix stuff
    float w = tan(0.5*cam_FOV);
    float h = w/cam_aspectRatio;
    mat4 proj_matrix = mat4(
      cam_minDist/w, 0.0, 0.0, 0.0,
      0.0, cam_minDist/h, 0.0, 0.0,
      0.0, 0.0, (cam_maxDist+cam_minDist)/(cam_maxDist-cam_minDist), 2.0*cam_maxDist*cam_minDist/(cam_maxDist-cam_minDist),
      0.0, 0.0, 1.0, 0.0
    );

    return proj_matrix*(cam_matrix*vec4(vtx_pos[i],1.0)); //apply camera translation and then projection matrix
}


#endif

#ifdef PIXEL
vec4 effect(vec4 color, Image tex, vec2 texture_coords, vec2 screen_coords){
    return color;
}
#endif
